@@ -18,16 +18,20 @@ class SugarJar
       SugarJar::Log.debug("Commands.initialize options: #{options}")
       @ghuser = options['github_user']
       @ghhost = options['github_host']
-      @cli = options['github_cli']
       @ignore_dirty = options['ignore_dirty']
       @ignore_prerun_failure = options['ignore_prerun_failure']
       @repo_config = SugarJar::RepoConfig.config
       @color = options['color']
+      @feature_prefix = options['feature_prefix']
       @checks = {}
       @main_branch = nil
       @main_remote_branches = {}
       return if options['no_change']
 
+      # technically this doesn't "change" things, but we won't have this
+      # option on the no_change call
+      @cli = determine_cli(options['github_cli'])
+
       set_hub_host
       set_commit_template if @repo_config['commit_template']
     end
@@ -35,6 +39,7 @@ class SugarJar
     def feature(name, base = nil)
       assert_in_repo
       SugarJar::Log.debug("Feature: #{name}, #{base}")
+      name = fprefix(name)
       die("#{name} already exists!") if all_local_branches.include?(name)
       base ||= most_main
       base_pieces = base.split('/')
@@ -49,6 +54,7 @@ class SugarJar
     def bclean(name = nil)
       assert_in_repo
       name ||= current_branch
+      name = fprefix(name) unless all_local_branches.include?(name)
       if clean_branch(name)
         SugarJar::Log.info("#{name}: #{color('reaped', :green)}")
       else
@@ -180,28 +186,34 @@ class SugarJar
 
       reponame = File.basename(repo, '.git')
       dir ||= reponame
+      org = extract_org(repo)
 
       SugarJar::Log.info("Cloning #{reponame}...")
 
       # GH's 'fork' command (with the --clone arg) will fork, if necessary,
       # then clone, and then setup the remotes with the appropriate names. So
       # we just let it do all the work for us and return.
-      if gh?
+      #
+      # Unless the repo is in our own org and cannot be forked, then it
+      # will fail.
+      if gh? && org != @ghuser
         ghcli('repo', 'fork', '--clone', canonicalize_repo(repo), dir, *args)
         SugarJar::Log.info('Remotes "origin" and "upstream" configured.')
         return
       end
 
-      # For 'hub', first we clone, using git, as 'hub' always needs a repo
-      # to operate on.
+      # For 'hub' first we clone, using git, as 'hub' always needs a repo to
+      # operate on.
+      #
+      # Or for 'gh' when we can't fork...
       git('clone', canonicalize_repo(repo), dir, *args)
 
       # Then we go into it and attempt to use the 'fork' capability
+      # or if not
       Dir.chdir dir do
         # Now that we have a repo, if we have a hub host set it.
         set_hub_host
 
-        org = extract_org(repo)
         SugarJar::Log.debug("Comparing org #{org} to ghuser #{@ghuser}")
         if org == @ghuser
           puts 'Cloned forked or self-owned repo. Not creating "upstream".'
@@ -339,6 +351,16 @@ class SugarJar
 
     private
 
+    def fprefix(name)
+      return name unless @feature_prefix
+
+      newname = "#{@feature_prefix}#{name}"
+      SugarJar::Log.debug(
+        "Munging feature name: #{name} -> #{newname} due to feature prefix",
+      )
+      newname
+    end
+
     def _smartpush(remote, branch, force)
       unless remote && branch
         remote ||= 'origin'
@@ -686,7 +708,7 @@ class SugarJar
       ).stdout.lines.reject do |line|
         line.start_with?('-')
       end
-      if out.length.zero?
+      if out.empty?
         SugarJar::Log.debug(
           "cherry-pick shows branch #{branch} obviously safe to delete",
         )
@@ -833,6 +855,29 @@ class SugarJar
       end
     end
 
+    def determine_cli(cli)
+      return cli if %w{gh hub}.include?(cli)
+
+      die("'github_cli' has unknown setting: #{cli}") unless cli == 'auto'
+
+      SugarJar::Log.debug('github_cli set to auto')
+
+      if which_nofail('gh')
+        SugarJar::Log.debug('Found "gh"')
+        return 'gh'
+      end
+      if which_nofail('hub')
+        SugarJar::Log.debug('Did not find "gh" but did find "hub"')
+        return 'hub'
+      end
+
+      die(
+        'Neither "gh" nor "hub" found in PATH, please ensure at least one ' +
+        'of these utilities is in the PATH. If both are available you can ' +
+        'specify which to use with --github-cli',
+      )
+    end
+
     def hub?
       @cli == 'hub'
     end
